local bit_band = bit.band
local string_match = string.match
local tinsert = table.insert
local tremove = table.remove
local GetSpellInfo = GetSpellInfo

local groupParts={}

local filters
local combatants


local function CreateName(ability, info, power, school, hitType, srcGUID, dstGUID)
	return string.format('%02x%02x%02x%05x%s%s%s', info, power, school, hitType, ability, srcGUID, dstGUID)
end

local function SplitName(name)
	local info, power, school, hitType, ability, srcGUID, dstGUID = string_match(name, '(%x%x)(%x%x)(%x%x)(%x%x%x%x%x)(.*)(0x.*)(0x.*)')
	info = tonumber(info, 16)
	power = tonumber(power, 16)
	school = tonumber(school, 16)
	hitType = tonumber(hitType, 16)

	return info, power, school, hitType, ability, srcGUID, dstGUID
end

local function AddData(dataTable, amount, resist, absorb, block, over, time)
	--First increment the count
	if dataTable.count==nil then
		dataTable.count = 1
	else
		dataTable.count = dataTable.count + 1
	end

	--Next if we have an amount then setup stats involving it
	if amount~=nil then
		if dataTable.total==nil then
			dataTable.total = amount
			dataTable.min = amount
			dataTable.max = amount
		else
			dataTable.total = dataTable.total + amount
			
			if dataTable.min>amount then
				dataTable.min = amount
			elseif dataTable.max<amount then
				dataTable.max = amount
			end
		end
	end

	if resist~=nil then
		if dataTable.resist==nil then
			dataTable.resist = resist
		else
			dataTable.resist = dataTable.resist + resist
		end
	end

	if absorb~=nil then
		if dataTable.absorb==nil then
			dataTable.absorb = absorb
		else
			dataTable.absorb = dataTable.absorb + absorb
		end
	end

	if block~=nil then
		if dataTable.block==nil then
			dataTable.block = block
		else
			dataTable.block = dataTable.block + block
		end
	end

	if over~=nil then
		if dataTable.over==nil then
			dataTable.over = over
		else
			dataTable.over = dataTable.over + over
		end
	end

	if time~=nil then
		if dataTable.time==nil then
			dataTable.time = time
		else
			dataTable.time = dataTable.time + time
		end
	end
end

local function MergeData(dataTable, count, amount, min, max, resist, absorb, block, over, time)
	--First increment the count
	if count~=nil then
		if dataTable.count==nil then
			dataTable.count = count
		else
			dataTable.count = dataTable.count + count
		end
	end

	--Next if we have an amount then setup stats involving it
	if amount~=nil then
		if dataTable.total==nil then
			dataTable.total = amount
		else
			dataTable.total = dataTable.total + amount
		end
	end

	if min~=nil and (dataTable.min==nil or dataTable.min>min) then
		dataTable.min = min
	end

	if max~=nil and (dataTable.max==nil or dataTable.max<max) then
		dataTable.max = max
	end

	if resist~=nil then
		if dataTable.resist==nil then
			dataTable.resist = resist
		else
			dataTable.resist = dataTable.resist + resist
		end
	end

	if absorb~=nil then
		if dataTable.absorb==nil then
			dataTable.absorb = absorb
		else
			dataTable.absorb = dataTable.absorb + absorb
		end
	end

	if block~=nil then
		if dataTable.block==nil then
			dataTable.block = block
		else
			dataTable.block = dataTable.block + block
		end
	end

	if over~=nil then
		if dataTable.over==nil then
			dataTable.over = over
		else
			dataTable.over = dataTable.over + over
		end
	end

	if time~=nil then
		if dataTable.time==nil then
			dataTable.time = time
		else
			dataTable.time = dataTable.time + time
		end
	end
end

--Combatant filter takes in a table with potential filters
-- required - does this require there be a combatant passed?
-- levelMin - Does a >= comparison with the combatant's level to ensure its greater than or equal
-- levelMax - Does a <= comparison with the combatant's level to ensure its less than or equal
-- class - a Table that if it exists requires the combatant's class be contained
-- race - a Table that if it exists requires the combatant's race be contained
-- classification - a Table that if it exists requires the combatant's class be contained
-- creatureType - a Table that if it exists requires the combatant's creature type be contained
-- creatureFamily - a Table that if it exists requires the combatant's creature family be contained
-- faction - a Table that if it exists requires the combatant's faction be contained
-- sex - a Table that if it exists requires the combatant's sex be contained
local function combatantFilter(filter, guid)
	if guid==nil then
		if filter.required then
			return false
		end
		return true
	end
	local attrTable = combatants[guid].attrTable
	return not (	(filter.levelMin and attrTable.level<filter.levelMin) or
			(filter.levelMax and attrTable.level>filter.levelMax) or
			(filter.class	and (attrTable.class==nil or filter.class[attrTable.class]==nil) ) or
			(filter.race	and (attrTable.race==nil  or filter.race[attrTable.race]==nil)   ) or
			(filter.classification and (attrTable.classification==nil or filter.classification[attrTable.classification]==nil)) or
			(filter.creatureType and (attrTable.creatureType==nil or filter.creatureType[attrTable.creatureType]==nil)) or
			(filter.faction and (attrTable.faction==nil or filter.faction[attrTable.faction]==nil)) or
			(filter.sex and (attrTable.sex==nil or filter.sex[attrTable.sex]==nil)) or
			(filter.trivial and (attrTable.trivial==nil or filter.trivial[attrTable.trivial or false]==nil)) or
			(filter.affiliation and filter.affiliation[attrTable.affiliation or "Others"]==nil) or
			(filter.reaction and filter.reaction[attrTable.reaction or "Neutral"]==nil) )
end
local function CheckForFilter(dataSet, srcGUID, dstGUID, kAbility, abilityTable, info, power, school, hitType)
	for filterID, filter in pairs(filters) do
		if filter.dataSet == dataSet then
			if not ((filter.info	and bit_band(info,	filter.info)	==0) or
			        (filter.power	and bit_band(power,	filter.power)	==0) or
			        (filter.school	and bit_band(school,	filter.school)	==0) or
			        (filter.hittype	and bit_band(hitType,	filter.hittype)	==0)) and
				(filter.srcCombatant==nil or combatantFilter(filter.srcCombatant, srcGUID)) and
				(filter.dstCombatant==nil or combatantFilter(filter.dstCombatant, dstGUID)) then
				local viewTable = filter.viewTable
				local groupOn = groupParts[filter.groupOn]

				--Have we added this GUID yet
				if viewTable[groupOn]==nil then
					viewTable[groupOn] = Recount2:GetTable()
				end

				--Should we combine? Lets not yet
				viewTable[groupOn][kAbility] = abilityTable
			end
		end
	end
end

local function AddForFilter(dataSet, kAbility, amount, resist, absorb, block, over, time) 
	for filterID, filter in pairs(filters) do
		if filter.dataSet == dataSet then
			local viewTable = filter.viewTable
			local groupOn = groupParts[filter.groupOn]
			--If these are in the view table then it must have passed the filter
			if viewTable[groupOn]~=nil and viewTable[groupOn][kAbility]~=nil then
				local viewCompress = filter.viewCompress[groupOn]
				if viewCompress==nil then
					viewCompress = Recount2:GetTable()
					filter.viewCompress[groupOn] = viewCompress
				end

				AddData(viewCompress, amount, resist, absorb, block, over, time)
				
				filter.groupUpdates[groupOn] = true
				filter.updateFunc(unpack(filter.updateArgs))
			end
		end
	end
end



function Recount2:AddMainData(timestamp, srcGUID, dstGUID, spellID, ability, info, power, school, hitType, amount, resist, absorb, block, over)
	if spellID then
		ability = spellID
	end

	--Grouping data
	local groupParts = groupParts

	groupParts.Info = info
	groupParts.Power = power
	groupParts.School = school
	groupParts["Hit Type"] = hitType
	groupParts.Ability = ability

	--Create the name that will be used
	local abilityName = CreateName(ability, info, power, school, hitType, srcGUID, dstGUID)

	--Set grouping parts for src
	groupParts["Source GUID"] = srcGUID
	local attrTable = combatants[srcGUID].attrTable
	groupParts["Source Name"] = attrTable.name or "No One"
	groupParts["Source Class"] = attrTable.class or "UNKNOWN"	
	groupParts["Source Level"] = attrTable.level or "Unknown"
	groupParts["Source Faction"] = attrTable.faction or "Unknown"
	groupParts["Source Race"] = attrTable.race or "Unknown"
	groupParts["Source Classification"] = attrTable.classification or "Unknown"
	groupParts["Source Creature Type"] = attrTable.creatureType or "Unknown"
	groupParts["Source Creature Family"] = attrTable.creatureFamily or "Unknown"
	groupParts["Source Sex"] = attrTable.sex or "Unknown"
	groupParts["Source Trivial"] = attrTable.trivial or "Unknown"

	--Set grouping parts for dst
	groupParts["Target GUID"] = dstGUID
	attrTable = combatants[dstGUID].attrTable
	groupParts["Target Name"] = attrTable.name or "No One"
	groupParts["Target Class"] = attrTable.class or "UNKNOWN"	
	groupParts["Target Level"] = attrTable.level or "Unknown"
	groupParts["Target Faction"] = attrTable.faction or "Unknown"
	groupParts["Target Race"] = attrTable.race or "Unknown"
	groupParts["Target Classification"] = attrTable.classification or "Unknown"
	groupParts["Target Creature Type"] = attrTable.creatureType or "Unknown"
	groupParts["Target Creature Family"] = attrTable.creatureFamily or "Unknown"
	groupParts["Target Sex"] = attrTable.sex or "Unknown"
	groupParts["Target Trivial"] = attrTable.trivial or "Unknown"
	
	--Determine Source's time contribution from this action (use 1.5 seconds as our window but get cast times)
	local lastActive = self.LastActive[srcGUID]
	local time
	if lastActive then
		time = timestamp - lastActive.time
		if lastActive.spellID==spellID then
			if time>10 then
				_, _, _, _, _, _, time = GetSpellInfo(spellID)
				if time==0 then
					time = 1.5
				else
					time = time*0.001
				end
			end
		--If we didn't have a start cast message must be at most GCD then
		elseif time>1.5 then
			if spellID and spellID~="Melee" then
				local castTime
				_, _, _, _, _, _, castTime = GetSpellInfo(spellID)
				castTime = castTime*0.001
				if castTime==0 then
					castTime=1.5
				end
				if time==nil or castTime<time then
					time = castTime
				end
			else
				time = 1.5
			end
		end
	else
		lastActive = Recount2:GetTable()
		self.LastActive[srcGUID] = lastActive
		if spellID and spellID~="Melee" then
			_, _, _, _, _, _, time = GetSpellInfo(spellID)
			if time==0 then
				time = 1.5
			else
				time = time*0.001
			end
		else
			time = 1.5
		end
	end

	lastActive.spellID = -1
	lastActive.time = timestamp

	local updatingViews = Recount2.updatingViews
	--Update the current active datasets
	for dataId, dataSet in pairs(self.activeDatasets) do
		dataTable = dataSet[abilityName]
		if dataTable==nil then
			dataTable = self:GetTable()
			dataSet[abilityName] = dataTable

			--Is this in a filter?
			if updatingViews then
				CheckForFilter(dataSet, srcGUID, dstGUID, abilityName, dataTable, info, power, school, hitType)
			end
		end

		--Add the data to the tables
		if updatingViews then
			AddForFilter(dataSet, abilityName, amount, resist, absorb, block, over, time)
		end
		AddData(dataTable, amount, resist, absorb, block, over, time)
	end
end

local function FilterCombatantData(filterID)
	--Should be using caches but for now we just want to ensure this all works
	if Recount2.viewTable[filterID]~=nil then
		Recount2:FreeTableRecurseLimit(Recount2.viewTable[filterID], 1)
	end
	Recount2.viewTable[filterID] = Recount2:GetTable()
	local viewTable = Recount2.viewTable[filterID]

	--Get our filter info now
	local filter = filters[filterID]

	--Make sure we have a filter
	if filter == nil then
		Recount2:Print("No filter set for ID", filterID)
		return
	end

	--Cache them out of the table so don't have to index
	local filter_dataSet	= filter.dataSet
	local filter_info	= filter.info
	local filter_power	= filter.power
	local filter_school	= filter.school
	local filter_hittype	= filter.hitType

	for kAbility, vAbility in pairs(filter_dataSet) do
		local info, power, school, hitType, ability, srcGUID, dstGUID = SplitName(kAbility)
		--Calculate if it passes the filter
		if not ((filter_info	and bit_band(info,	filter_info)	==0) or
			(filter_power	and bit_band(power,	filter_power)	==0) or
			(filter_school	and bit_band(school,	filter_school)	==0) or
			(filter_hittype	and bit_band(hitType,	filter_hittype)	==0)) and
			(filter.srcCombatant==nil or combatantFilter(filter.srcCombatant, srcGUID)) and
			(filter.dstCombatant==nil or combatantFilter(filter.dstCombatant, dstGUID)) then
			
			groupParts.Info, groupParts.Power, groupParts.School, groupParts["Hit Type"], groupParts.Ability, groupParts.Who = info, power, school, hitType, ability, kGUID

			--Set grouping parts for src
			groupParts["Source GUID"] = srcGUID
			local attrTable = combatants[srcGUID].attrTable
			groupParts["Source Name"] = attrTable.name or "No One"
			groupParts["Source Class"] = attrTable.class or "UNKNOWN"	
			groupParts["Source Level"] = attrTable.level or "Unknown"
			groupParts["Source Faction"] = attrTable.faction or "Unknown"
			groupParts["Source Race"] = attrTable.race or "Unknown"
			groupParts["Source Classification"] = attrTable.classification or "Unknown"
			groupParts["Source Creature Type"] = attrTable.creatureType or "Unknown"
			groupParts["Source Creature Family"] = attrTable.creatureFamily or "Unknown"
			groupParts["Source Sex"] = attrTable.sex or "Unknown"
			groupParts["Source Trivial"] = attrTable.trivial or "Unknown"


			--Set grouping parts for dst
			groupParts["Target GUID"] = dstGUID
			attrTable = combatants[dstGUID].attrTable
			groupParts["Target Name"] = attrTable.name or "No One"
			groupParts["Target Class"] = attrTable.class or "UNKNOWN"	
			groupParts["Target Level"] = attrTable.level or "Unknown"
			groupParts["Target Faction"] = attrTable.faction or "Unknown"
			groupParts["Target Race"] = attrTable.race or "Unknown"
			groupParts["Target Classification"] = attrTable.classification or "Unknown"
			groupParts["Target Creature Type"] = attrTable.creatureType or "Unknown"
			groupParts["Target Creature Family"] = attrTable.creatureFamily or "Unknown"
			groupParts["Target Sex"] = attrTable.sex or "Unknown"
			groupParts["Target Trivial"] = attrTable.trivial or "Unknown"


			local groupOn = groupParts[filter.groupOn]
			--Have we added this GUID yet
			if viewTable[groupOn]==nil then
				viewTable[groupOn] = Recount2:GetTable()
			end

			--Should we combine? Lets not yet
			viewTable[groupOn][kAbility] = vAbility
		end
	end
end

local CompressedTables = {}

local function CompressFilter(filterID)
	--The objective here is to compress all the stats into a single set of stats per combatant

	--Should be using caches but for now we just want to ensure this all works
	local viewCompress = Recount2.viewCompress[filterID]
	if viewCompress~=nil then
		for k, v in pairs(viewCompress) do
			tinsert(CompressedTables, v)
			viewCompress[k] = nil
		end
	else
		viewCompress = Recount2:GetTable()
		Recount2.viewCompress[filterID] = viewCompress 
	end
	
	local viewTable = Recount2.viewTable[filterID]
	local groupUpdates = Recount2.filter[filterID].groupUpdates

	--Make sure we have a filtered table
	if viewTable == nil then
		Recount2:Print("No filtered table for ID", filterID)
		return
	end

	for key, vCombatant in pairs(viewTable) do
		local initTable
		if #CompressedTables>0 then
			viewCompress[key] = tremove(CompressedTables)
			initTable = true
		else
			viewCompress[key] = Recount2:GetTable()
			initTable = false
		end
		local compressedData = viewCompress[key]

		groupUpdates[key] = true
		for kAbility, vAbility in pairs(vCombatant) do
			--Damn min & max prevent us from doing this with a table whatever guess its faster this way if a little messier
			--Though might consider in the future allowing this stuff to be setup by specifying functions for each argument and it uses a lookup table for them
			--Then again prolly only do something like that for nonMainData
			if vAbility.count~=nil then
				if compressedData.count==nil or initTable then
					compressedData.count = vAbility.count
				else
					compressedData.count = compressedData.count + vAbility.count
				end
			elseif initTable then
				compressedData.count = nil
			end

			if vAbility.total~=nil then
				if compressedData.total==nil or initTable then
					compressedData.total = vAbility.total
				else
					compressedData.total = compressedData.total + vAbility.total
				end
			elseif initTable then
				compressedData.total = nil
			end

			if vAbility.min~=nil then
				if compressedData.min==nil or initTable then
					compressedData.min = vAbility.min
				else
					if compressedData.min>vAbility.min then
						compressedData.min = vAbility.min
					end
				end
			elseif initTable then
				compressedData.min = nil
			end

			if vAbility.max~=nil then
				if compressedData.max==nil or initTable then
					compressedData.max = vAbility.max
				else
					if compressedData.max<vAbility.max then
						compressedData.max = vAbility.max
					end
				end
			elseif initTable then
				compressedData.max = nil
			end

			if vAbility.resist~=nil then
				if compressedData.resist==nil or initTable then
					compressedData.resist = vAbility.resist
				else
					compressedData.resist = compressedData.resist + vAbility.resist
				end
			elseif initTable then
				compressedData.resist = nil
			end

			if vAbility.absorb~=nil then
				if compressedData.absorb==nil or initTable then
					compressedData.absorb = vAbility.absorb
				else
					compressedData.absorb = compressedData.absorb + vAbility.absorb
				end
			elseif initTable then
				compressedData.absorb = nil
			end

			if vAbility.block~=nil then
				if compressedData.block==nil or initTable then
					compressedData.block = vAbility.block
				else
					compressedData.block = compressedData.block + vAbility.block
				end
			elseif initTable then
				compressedData.block = nil
			end

			if vAbility.over~=nil then
				if compressedData.over==nil or initTable then
					compressedData.over = vAbility.over
				else
					compressedData.over = compressedData.over + vAbility.over
				end
			elseif initTable then
				compressedData.over = nil
			end

			if vAbility.time~=nil then
				if compressedData.time==nil or initTable then
					compressedData.time = vAbility.time
				else
					compressedData.time = compressedData.time + vAbility.time
				end
			elseif initTable then
				compressedData.time = nil
			end


			initTable = false
		end
	end

	Recount2.filter[filterID].updateFunc(unpack(Recount2.filter[filterID].updateArgs))

	for k, v in ipairs(CompressedTables) do
		Recount2:FreeTable(v)
		CompressedTables[k] = nil
	end
end



local function CompareTables(a, b)
	for k, v in pairs(a) do
		if type(v)=="table" and type(b[k])=="table" then
			if not CompareTables(v, b[k]) then
				return false
			end
		elseif b[k]~=v then
			return false
		end
	end

	for k, v in pairs(b) do
		if type(v)=="table" and type(a[k])=="table" then
			if not CompareTables(v, a[k]) then
				return false
			end
		elseif a[k]~=v then
			return false
		end
	end

	return true
end

function Recount2:SetFilter(filterID, dataSet, srcCombatant, dstCombatant, info, power, school, hitType, groupOn, updateFunc, ...)
	local filter = filters[filterID]
	if filter == nil then
		filter = self:GetTable()
		filters[filterID] = filter
		filter.groups = self:GetTable()
	end

	if groupOn==nil then
		groupOn = "Source GUID"
	end

	if filter.dataSet and filter.dataSet == dataSet and
	   ((info==nil	  and filter.info==nil)	   or (filter.info and filter.info == info)) and
	   ((power==nil	  and filter.power==nil)   or (filter.power and filter.power == power)) and
	   ((school==nil  and filter.school==nil)  or (filter.school and filter.school == school)) and
	   ((hitType==nil and filter.hitType==nil) or (filter.hitType and filter.hitType == hitType)) and
	   (groupOn == filter.groupOn) and
	   ((srcCombatant==nil and filter.srcCombatant==nil) or (srcCombatant and filter.srcCombatant and CompareTables(srcCombatant, filter.srcCombatant) )) and
	   ((dstCombatant==nil and filter.dstCombatant==nil) or (dstCombatant and filter.dstCombatant and CompareTables(dstCombatant, filter.dstCombatant) )) then
		--Filter is already set don't need to set it again and refilter
		--Though we do need to reset that they need updates
		local viewTable = Recount2.viewTable[filterID]
		local groupUpdates = filter.groupUpdates
		
		for k in pairs(viewTable) do
			groupUpdates[k] = true
		end
		filter.updateFunc(unpack(filter.updateArgs))
		return
	end
	
	filter.dataSet = dataSet
	filter.info = info
	filter.power = power
	filter.school = school
	filter.hitType = hitType
	filter.srcCombatant = srcCombatant
	filter.dstCombatant = dstCombatant
	

	filter.groupUpdates = self:GetTable()
	filter.groupOn = groupOn
	filter.updateFunc = updateFunc

	local args = self:GetTable()
	local nArgs = select("#", ...)

	for i = 1, nArgs do
		local curArg = select(i, ...)
		tinsert(args, curArg)
	end

	filter.updateArgs = args

	FilterCombatantData(filterID)
	CompressFilter(filterID)

	filter.viewTable = Recount2.viewTable[filterID]
	filter.viewCompress = Recount2.viewCompress[filterID]
end

function Recount2:ClearFilter(filterID)
	if filters[filterID] then
		Recount2:FreeTable(filters[filterID])
		filters[filterID] = nil

		Recount2:FreeTableRecurseLimit(Recount2.viewTable[filterID], 1)
		Recount2.viewTable[filterID] = nil
		
		Recount2:FreeTableRecurse(Recount2.viewCompress[filterID])
		Recount2.viewCompress[filterID] = nil
	end
end

function Recount2:ResetFilters()
	for filterID in pairs(self.filter) do
		Recount2:FreeTableRecurseLimit(Recount2.viewTable[filterID], 1)
		Recount2.viewTable[filterID] = Recount2:GetTable()
		
		Recount2:FreeTableRecurse(Recount2.viewCompress[filterID])
		Recount2.viewCompress[filterID] = Recount2:GetTable()
	end
end

--Grouping
function Recount2:GroupTables(data, groupOn)
	local groupParts = groupParts
	results = Recount2:GetTable()
	local srcGUID, dstGUID

	for name, value in pairs(data) do
		groupParts.Info, groupParts.Power, groupParts.School, groupParts["Hit Type"], groupParts.Ability, srcGUID, dstGUID = SplitName(name)

		--Set grouping parts for src
		groupParts["Source GUID"] = srcGUID
		local attrTable = combatants[srcGUID].attrTable
		groupParts["Source Name"] = attrTable.name or "No One"
		groupParts["Source Class"] = attrTable.class or "UNKNOWN"	
		groupParts["Source Level"] = attrTable.level or "Unknown"
		groupParts["Source Faction"] = attrTable.faction or "Unknown"
		groupParts["Source Race"] = attrTable.race or "Unknown"
		groupParts["Source Classification"] = attrTable.classification or "Unknown"
		groupParts["Source Creature Type"] = attrTable.creatureType or "Unknown"
		groupParts["Source Creature Family"] = attrTable.creatureFamily or "Unknown"
		groupParts["Source Sex"] = attrTable.sex or "Unknown"
		groupParts["Source Trivial"] = attrTable.trivial or "Unknown"


		--Set grouping parts for dst
		groupParts["Target GUID"] = dstGUID
		attrTable = combatants[dstGUID].attrTable
		groupParts["Target Name"] = attrTable.name or "No One"
		groupParts["Target Class"] = attrTable.class or "UNKNOWN"	
		groupParts["Target Level"] = attrTable.level or "Unknown"
		groupParts["Target Faction"] = attrTable.faction or "Unknown"
		groupParts["Target Race"] = attrTable.race or "Unknown"
		groupParts["Target Classification"] = attrTable.classification or "Unknown"
		groupParts["Target Creature Type"] = attrTable.creatureType or "Unknown"
		groupParts["Target Creature Family"] = attrTable.creatureFamily or "Unknown"
		groupParts["Target Sex"] = attrTable.sex or "Unknown"
		groupParts["Target Trivial"] = attrTable.trivial or "Unknown"


		
		grouping = groupParts[groupOn]
		if results[grouping]==nil then
			results[grouping]=Recount2:GetTable()
			results[grouping].tables = Recount2:GetTable()
		end

		MergeData(results[grouping], value.count, value.total, value.min, value.max, value.resist, value.absorb, value.block, value.over)
		results[grouping].tables[name] = value
	end

	return results
end


--Performance Modes
function Recount2:StopViewUpdating()
	Recount2.updatingViews = false
end

function Recount2:StartViewUpdating()
	if Recount2.updatingViews==false then
		Recount2.updatingViews = true
		self:UpdateViews()
	end
end

function Recount2:UpdateViews()
	for filterID in pairs(self.filter) do
		FilterCombatantData(filterID)
		CompressFilter(filterID)
	end

	for _, window in pairs(self.mainListWindows) do
		if window:IsShown() then
			window.listFrame:Refresh()
		end
	end
end

--Combat Status
local function EnterCombat(self, event)
	Recount2.inCombat = true
	if Recount2.options.performanceMode then
		Recount2:StopViewUpdating()
	end
end

local function ExitCombat(self, event)
	Recount2.inCombat = false
	if Recount2.options.performanceMode then
		Recount2:StartViewUpdating()
	end
end

function Recount2:AddActiveDataset(id, dataset)
	self.activeDatasets[id] = dataset
end

function Recount2:RemoveActiveDataset(id)
	self.activeDatasets[id] = nil
end

function Recount2:GetActiveDataset(id)
	return self.activeDatasets[id]
end

--Data Init
function Recount2:InitDataRecorder()
	filters = self.filter
	combatants = self.Data.Combatants

	self.inCombat = UnitAffectingCombat("player")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", EnterCombat)
	self:RegisterEvent("PLAYER_REGEN_ENABLED", ExitCombat)
	self.updatingViews = not self.options.performanceMode or not self.inCombat
end
